@@ -9,7 +9,7 @@ use Mojo::Util;
 
 use constant DEBUG => $ENV{MOJO_OPENAPI_DEBUG} || 0;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 my $BASE = __PACKAGE__;
 
@@ -23,28 +23,22 @@ has base_url => sub {
 
 has ua => sub { Mojo::UserAgent->new };
 
-sub local_app {
-  my ($self, $app) = @_;
-  my $ua = $self->ua;
-
-  $ua->ioloop(Mojo::IOLoop->singleton);
-  $ua->server->app($app);
-  $self->base_url->host($ua->server->url->host);
-  $self->base_url->port($ua->server->url->port);
-
-  return $self;
-}
-
 sub new {
-  my ($class, $url) = (shift, shift);
+  my ($class, $specification) = (shift, shift);
   my $attrs = @_ == 1 ? shift : {@_};
   my $validator = JSON::Validator::OpenAPI::Mojolicious->new;
 
-  $class = $class->_url_to_class($url);
-  _generate_class($class, $validator->load_and_validate_schema($url, $attrs)) unless $class->isa($BASE);
+  $validator->ua->server->app($attrs->{app}) if $attrs->{app};
+  $class = $class->_url_to_class($specification);
+  _generate_class($class, $validator->load_and_validate_schema($specification, $attrs)) unless $class->isa($BASE);
 
   my $self = bless $attrs, $class;
-  $self->ua->transactor->name('Mojo-OpenAPI (Perl)');
+  $self->ua->transactor->name('Mojo-OpenAPI (Perl)') unless $self->{ua};
+
+  if (my $app = delete $self->{app}) {
+    $self->base_url->host(undef)->scheme(undef)->port(undef);
+    $self->ua->server->app($app);
+  }
 
   return $self;
 }
@@ -99,7 +93,7 @@ sub _generate_method {
 }
 
 sub _generate_tx {
-  my ($self, $http_method, $path_spec, $op_spec, $params) = @_;
+  my ($self, $http_method, $path_spec, $op_spec, $params, %args) = @_;
   my $v   = $self->_validator;
   my $url = $self->base_url->clone;
   my (%headers, %req, @body, @errors);
@@ -110,6 +104,10 @@ sub _generate_tx {
     my ($in, $name, $type) = @$p{qw(in name type)};
     my @e;
 
+    if ($in eq 'body' and exists $args{body}) {
+      $params->{$name} = $args{body};
+    }
+
     if (exists $params->{$name} or $p->{required}) {
       @e = $v->validate($params,
         {type => 'object', required => $p->{required} ? [$name] : [], properties => {$name => $p->{schema} || $p}});
@@ -187,8 +185,8 @@ appreciated.
 
 =head2 Open API specification
 
-The input C<url> given to L</new> need to point to a valid OpenAPI document, in
-either JSON or YAML format. Example:
+The specification given to L</new> need to point to a valid OpenAPI document,
+in either JSON or YAML format. Example:
 
   ---
   swagger: 2.0
@@ -208,7 +206,7 @@ either JSON or YAML format. Example:
 
 C<host>, C<basePath> and the first item in C<schemes> will be used to construct
 L</base_url>. This can be altered at any time, if you need to send data to a
-custom URL.
+custom endpoint.
 
 =head2 Client
 
@@ -261,8 +259,9 @@ L<Mojolicious::Lite> application given as argument. (Useful for testing)
 
 =head2 new
 
-  $client = OpenAPI::Client->new($specification, %attrs);
   $client = OpenAPI::Client->new($specification, \%attrs);
+  $client = OpenAPI::Client->new($specification, %attrs);
+  $client = OpenAPI::Client->new($specification, app => Mojolicious->new);
 
 Returns an object of a generated class, with methods generated from the Open
 API specification located at C<$specification>. See L<JSON::Validator/schema>
@@ -271,6 +270,9 @@ for valid versions of C<$specification>.
 Note that the class is cached by perl, so loading a new specification from the
 same URL will not generate a new class.
 
+Specifying an C<app> is useful when running against a local L<Mojolicious>
+instance.
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (C) 2017, Jan Henning Thorsen
